﻿using System;
using System.Linq;
using Microsoft.BusinessData.MetadataModel;
using Microsoft.BusinessData.MetadataModel.Collections;
using Microsoft.BusinessData.Runtime;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.BusinessData.SharedService;

using Microsoft.SharePoint.WebControls;

namespace BANF.Layouts.BANF
{
    public partial class BANFRelease : LayoutsPageBase
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (Request.QueryString["list"] != null && Request.QueryString["items"] != null && Request.QueryString["filter"] != null)
            {
                //parsing the query string
                string list = Request.QueryString["list"];
                string[] items = Request.QueryString["items"].ToString().Split(',');
                string[] filter = Request.QueryString["filter"].ToString().Split(',');
                
                //finding the list
                //Guid listGuid = new Guid(list);
                SPWeb web = SPContext.Current.Web;
                //SPList banfList = web.Lists[listGuid];
                SPList banfList = web.Lists["BANFList"];

                //constructing the CAML query
                SPQuery query = new SPQuery();
                string whereClause = "";
                foreach (string bdcId in items)
                {
                    whereClause += "<Eq><FieldRef Name=\"BdcIdentity\"/>" +
                                       "<Value Type=\"Text\">" + bdcId + "</Value>" +
                                   "</Eq>";
                }
                if (items.Length > 1)
                {
                    whereClause = "<Or>" + whereClause + "</Or>";
                }
                query.ViewXml = "<View>" +
                                   "<Method Name=\"ReadList\">" +
                                      "<Filter Name=\"Limit\" Value=\"500\"/>" +
                                      "<Filter Name=\"ItemsForRelease\" Value=\"" + filter[0] + "\"/>" +
                                      "<Filter Name=\"ReleaseCode\" Value=\"" + filter[1] + "\"/>" +
                                      "<Filter Name=\"ReleaseGroup\" Value=\"" + filter[2] + "\"/>" +
                                   "</Method>" +
                                   "<Query>" +
                                      "<Where>" + whereClause + "</Where>" +
                                   "</Query>" +
                                   "<ViewFields>" +
                                     "<FieldRef Name=\"RequisitionNumber\"/>" +
                                     "<FieldRef Name=\"RequisitionItem\"/>" +
                                     "<FieldRef Name=\"Material\"/>" +
                                     "<FieldRef Name=\"MaterialDescription\"/>" +
                                     "<FieldRef Name=\"Quantity\"/>" +
                                     "<FieldRef Name=\"Unit\"/>" +
                                   "</ViewFields>" +
                                "</View>";

                //executing the CAML query
                SPListItemCollection listItems = banfList.GetItems(query);

                //reading list item values - single selection
                if (listItems != null) {
                    string reqNumber = listItems[0]["RequisitionNumber"].ToString();
                    string reqItem = listItems[0]["RequisitionItem"].ToString();
                    string material = listItems[0]["Material"].ToString();
                    string description = listItems[0]["MaterialDescription"].ToString();
                    string qty = listItems[0]["Quantity"].ToString();
                    string unit = listItems[0]["Unit"].ToString();


                    //displaying item (single selection)
                    tbxRequisitionNumber.Text = reqNumber;
                    tbxRequisitionItem.Text = reqItem;
                    tbxMaterial.Text = material + " - " + description;
                    tbxMaterial.Width = (tbxMaterial.Text.Length * 5)+10;
                    tbxQty.Text = qty + " " + unit;
                    tbxQty.Width = (tbxQty.Text.Length * 5)+5;

                    //getting additional SAP release details
                    Get_SAP_Release_Info(reqNumber, reqItem);
                }
            }
        }

        protected void Get_SAP_Release_Info(string preq_no, string preq_item)
        {
            using (ERPConnectServicesContext context = new ERPConnectServicesContext())
            {
                GENERAL_RELEASE_INFOTable rel_info_table = new GENERAL_RELEASE_INFOTable();
                RELEASE_ALREADY_POSTEDTable rel_posted_table = new RELEASE_ALREADY_POSTEDTable();
                RELEASE_FINALTable rel_final_table = new RELEASE_FINALTable();
                RETURNTable6 bapi_ret_relinfo = context.BAPI_REQUISITION_GETRELINFO(preq_item, preq_no, ref rel_info_table, ref rel_posted_table, ref rel_final_table);

                RELEASE_FINALStructure rel_final_item = rel_final_table.Where(f => f.PREQ_ITEM == preq_item).FirstOrDefault();

                lblRelCodeText1.Text = rel_final_item.REL_CD_TX1;
                lblRelCodeText2.Text = rel_final_item.REL_CD_TX2;
                lblRelCodeText3.Text = rel_final_item.REL_CD_TX3;


                if (rel_posted_table.RowCount == 0)
                {
                    //nothing released yet
                    lblStatus1.CssClass = "banfLabelStatusReady";
                    lblStatus1.ToolTip = "Ready";
                    
                }
                else
                {
                    RELEASE_ALREADY_POSTEDStructure rel_posted_item = rel_posted_table.Where(Page => Page.PREQ_ITEM == preq_item).FirstOrDefault();
                    string[] prev_rel_codes = new string[8] { rel_posted_item.REL_CODE1, rel_posted_item.REL_CODE2, rel_posted_item.REL_CODE3, rel_posted_item.REL_CODE4, rel_posted_item.REL_CODE5, rel_posted_item.REL_CODE6, rel_posted_item.REL_CODE7, rel_posted_item.REL_CODE8 };

                    int i = 0;
                    while (prev_rel_codes[i] != String.Empty)
                    {
                        i++;
                    }
                    switch (i)
                    {
                        case 1:
                            lblStatus1.CssClass = "banfLabelStatusReleased";
                            lblStatus1.ToolTip = "Released";
                            lblStatus2.CssClass = "banfLabelStatusReady";
                            lblStatus2.ToolTip = "Ready";
                            break;
                        case 2:
                            lblStatus1.CssClass = "banfLabelStatusReleased";
                            lblStatus1.ToolTip = "Released";
                            lblStatus2.CssClass = "banfLabelStatusReleased";
                            lblStatus2.ToolTip = "Released";
                            lblStatus3.CssClass = "banfLabelStatusReady";
                            lblStatus3.ToolTip = "Ready";
                            break;
                        //case 3:.....
                    }
                }
                GENERAL_RELEASE_INFOStructure rel_info_item = rel_info_table.Where(i => i.PREQ_ITEM == preq_item).FirstOrDefault();
                tbxReleaseIndicatorCode.Text = rel_info_item.REL_IND;
                tbxReleaseIndicatorText.Text = rel_info_item.REL_IND_TX;
            }
        }

        protected void btnRelease_Click(Object sender, EventArgs e)
        {
            const string entityName = "BANF";
            const string lobSystemName = "BdcModelBANF";
            const string entityNamespace = "BANF.BdcModelBANF";
            using (SPSite site = new SPSite(SPContext.Current.Web.Url))
            {
                using (new Microsoft.SharePoint.SPServiceContextScope(SPServiceContext.GetContext(site)))
                {
                    lblMessage.Text = "Purchase Requisition is being released...";
                    try
                    {
                        BdcService service = SPFarm.Local.Services.GetValue<BdcService>(String.Empty);
                        IMetadataCatalog catalog = service.GetDatabaseBackedMetadataCatalog(SPServiceContext.Current);
                        IEntity entity = catalog.GetEntity(entityNamespace, entityName);
                        ILobSystemInstance lobSystemInstance = entity.GetLobSystem().GetLobSystemInstances()[0].Value;
                        //obtaining the method instance
                        IMethodInstance releaseInstance = entity.GetMethodInstance("Release", MethodInstanceType.GenericInvoker);
                        IParameterCollection parameters = releaseInstance.GetMethod().GetParameters();
                        Object[] rawParameters = new Object[parameters.Count];
                        rawParameters[0] = tbxRequisitionNumber.Text;
                        rawParameters[1] = tbxRequisitionItem.Text;
                        entity.Execute(releaseInstance, lobSystemInstance, ref rawParameters);
                        lblMessage.Text = "Purchase Requisition released successfully.";
                        btnRelease.Visible = false;
                    }
                    catch
                    {
                        lblMessage.Text = "Error during Release of Purchase Requisition.";
                        btnRelease.Visible = false;
                    }
                }
            }

        }

    }
}
